'use strict'

var co = require('co')

var koa = require('koa')
var logger = require('koa-logger')
var compress = require('koa-compress')
var validate = require('koa-validate')

var routerFactory = require('./routes/index.js')

var initApp = co.wrap(function * (config) {
  var app = koa()
  // MIDDLEWARE
  app.use(logger())
  app.use(validate())
  app.use(compress())

  // ROUTING
  let router = routerFactory(config)
  app.use(router.routes())
  app.use(router.allowedMethods())

  return app
})

var startService = function (config) {
  co(function * () {
    var app = yield initApp(config)
    // LISTEN
    app.listen(config.port, function () {
      console.log('App listening on port', config.port)
    })
  }).catch((error) => {
    console.log('=== INIT ERROR')
    console.log(error)
  })
}

// EXPORT
module.exports = {
  initApp: initApp,
  startService: startService
}
